/*
新作情報チェッカーWidget (Amazon.co.jpを使用)
2011-09-10版

2005.05.07~
written by SAITOH Daisuke(E)
mailto:dsaitoh@mac.com

●機能概要
　Amazon.co.jpのWebサービスを利用して、設定したキーワードで引っかかる商品を
発売日が新しい順に10件まで表示します。また、表示されたタイトルをクリックすると、
わたしのアソシエイトタグ付きでAmazonの商品ページを表示します。

なお、情報の更新は、
・Mac起動時
・検索キーワード設定時
・６時間ごと
のタイミングで行います。

●AmazonのWebサービスについて
　このWidgetでは「ItemSearch」機能を使っています。
　2009-08-15のAPI仕様変更時に独自PlugInを内蔵しました。Amazon規約によりこのPlugIn単独での利用およびリバースエンジニアリングは禁止いたします。

●更新履歴
2005-05-10
        データ取得ロジックを書き直し。出版日を表示。OR検索に対応。キャプション表示を追加。
2005-05-11
        出版日が含まれないデータがヒットするとフリーズする不具合を修正。
        初回ロード時に設定が存在したら、ランダムに１分以内に更新を実行。
2005-05-12
        書籍に加えて、音楽とDVDの検索を追加。
        OR時に重複した検索結果を排除するよう修正。
        内部構造を整理して今後に使い回せそうなルーチンをWidgetUtils.jsに切り出し。
2005-05-16
        Widget名を「新刊情報チェック」から「New Release」に変更。
        検索条件仕様を全面改訂。ジャンル・検索項目・検索キーワード・除外キーワードを５セット用意。また、未来日付の足切りも追加。
        内部処理の最適化。
2005-08-11
        データ通信を非同期処理に変更。データ取得中に止まってしまう不具合対策。
        同時に排他ロック処理を削除。（ちゃんと動いてなかった・・・）

2005-09-04
        Appleサイト掲載用に調整。
        Amazonアソシエイト表記を記載ルールにそって修正。

2005-11-20
        情報取得間隔を６時間に変更。
        情報更新間隔チェック方式を変更し、５分置きに調べるように変更。これによりスリープに対応。
        起動時の検索開始処理にランダムウェイトを挿入。（負荷の低減）
        前面／背面の画像修正。
        ステータスに最終更新日、エラー情報を表示するよう修正。
        Amazonアソシエイトタグを修正。

2006-10-01
        ジャンルに「クラシック」「ゲーム」を追加。
        
2009-08-11
        AmazonのAPI仕様変更に対応するためObjective-C製のPlugInを追加。
        ジャンル「おもちゃ」「エレクトロニクス」「コンピューター」「キッチン」を追加。
        各ジャンルのタイプに「タイトル」検索を追加。
        ジャンル「DVD」を「DVD/Blu-ray」に変更。
        背景色を選択式（５色）に変更。
        JavaScriptでのメモリリークを少なめになるよう調整。

2009-08-23
        プラグインをOSX10.4でも動作するよう修正。

2011-03-21
        内部構造の調整。
        クラシックが最新順に表示されない不具合を修正。
        ネットワーク状態監視を追加。更新実施チェック間隔を１分に短縮。
        日付情報が得られないジャンルはセールスランクで取得／表示するよう修正。
        .Macの同期に対応。
        Nintendo 3DS追加をはじめ、ジャンルの追加／整理。

2011-07-21
        Lionで表示が乱れたためCSSを調整。
        
2012-06-17
        アマゾン新仕様（検索結果取得上限ページ10まで）に対応。
        APIを2011-08-01に変更。
        内部構造を整理し、.co.jpと.comに対応。
*/

var __request = null;
var __requestSuccessHandler;
var __requestErrorHandler;
var __requestOfflineHandler;
var __dataLastUpdateDate;    //最終更新日時保持用
var __onlined = null;
var __successed = null;


/*=============================================================================
セットアップ処理

概要
    本widgetのスタートポイント。HTMLのonloadから呼び出すこと。
引数
    なし
戻値
    なし
=============================================================================*/
function load()
{
    //Dashcode環境の初期化
    dashcode.setupParts();
    //プラグインの初期化
    NewReleasePlugIn.setApiInfo(__const_aws_domain, __const_aws_path, __const_aws_version);

    //画面の初期化
    _load_data(true);
}

/*=============================================================================
データロード処理

概要
    保存してあるデータをロードする。
引数
    aRefesh : 更新実行ならtrueを指定。
戻値
    なし
=============================================================================*/
function _load_data(aRefresh)
{
var i;

    //画面の初期化
    //※情報を取り出し
    var caption = _loadConditionOfCaption();
    var limit = _loadConditionOfLimit();
    var color = _loadConditionOfColor();
    var genre_array = _loadConditionOfGenreArray();
    var srchtype_array = _loadConditionOfSrchTypeArray();
    var keyword_array = _loadConditionOfKeywordArray();
    var uncsy_array = _loadConditionOfExcludeArray();
    var html_array = _loadDataArray();
    //※未設定時は初期値を適用
    if ( null == caption ) caption = "";
    if ( null == limit ) limit = __const_default_limit;
    if ( null == color ) color = __const_default_color;
    //※表面表示
    _getElement("frontImage").className = color;
    _getElement("goAmazon").className = color;
    _getElement("limitSelect").value = limit;
    _getElement("captionArea").innerText = caption;
    _getElement("goAmazon").innerText = __const_label_go_amazon;
    _displayStatus(__const_label_status_default);
    _displayData(html_array);
    //※裏面表示
    _getElement("aboutThisWidget").innerText = __const_label_about;
    _getElement("useAssosiateLabel").innerText = __const_label_use_assosiate;
    _getElement("captionEdit").value = caption;
    _getElement("limitSelect").value = limit;
    _getElement("colorSelect").value = color;
    for ( i = 0 ; i < __const_condition_count ; i++ )
    {
        _writeGenreList(i);
        _getElement("genreSelect"+i).value = genre_array[i];
        _writeSrchTypeList(i, genre_array[i]);
        _getElement("srchtypeSelect"+i).value = srchtype_array[i];
        if ( keyword_array[i] )    _getElement("keywordEdit"+i).value = keyword_array[i];
        if ( uncsy_array[i] ) _getElement("excludeEdit"+i).value = uncsy_array[i];
    }

    //最終更新日時を初期化
    __dataLastUpdateDate = new Date();
    __dataLastUpdateDate.setTime( 0 );

    //更新指定で、キーワードが存在していたら、検索を実行
    if ( aRefresh )
    {
        var flag = false;
        for ( i = 0 ; i < keyword_array.length ; i++ )
        {
            if ( keyword_array[i] )
            {
                flag = true;
                break;
            }
        }
        if ( flag )
        {
            //ランダムで20秒以内のウェイトを入れる。
            _displayStatus( __const_msg_updating_standby, 1 );
            setTimeout( "_refreshList();", Math.floor( Math.random() * __const_random_interval_base + __const_wait_interval ) );
        }
    }
}


/*=============================================================================
widget破棄時処理

概要
    widgetがメモリから破棄される際のイベント。システムの終了では呼ばれない。
    Dashboard環境からユーザー操作（Xボタン）によって閉じるときに呼ばれる。
引数
    なし
戻値
    なし
=============================================================================*/
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //保存データをクリア
    _clearConditionArrays();
    _clearDataArray();
}


/*=============================================================================
widget非表示時処理

概要
    widgetが非表示となる際のイベント。
引数
    なし
戻値
    なし
=============================================================================*/
function hide()
{
    // Stop any timers to prevent CPU usage
}


/*=============================================================================
widget表示時処理

概要
    widgetが表示される際のイベント。
引数
    なし
戻値
    なし
=============================================================================*/
function show()
{
    // Restart any timers that were stopped on hide
}


/*=============================================================================
.Macとの同期時処理

概要
    .Macでの同期処理が実行されたとき。
引数
    なし
戻値
    なし
=============================================================================*/
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
    _load_data(false);
}


/*=============================================================================
オフライン時処理

概要
    ネットワークが切断されたときの処理
引数
    event : イベント情報
戻値
    なし
=============================================================================*/
function offline(event)
{
    __onlined = false;
    _displayStatus( null, 2 );
}

/*=============================================================================
オンライン時処理

概要
    ネットワークが接続されたときの処理
引数
    event : イベント情報
戻値
    なし
=============================================================================*/
function online(event)
{
    __onlined = true;
    _displayStatus( null, 0 );
}

/*=============================================================================
うら面の表示処理

概要
    うら面を表示する
引数
    event : イベント情報
戻値
    なし
=============================================================================*/
function showBack(event)
{
    var front = _getElement("front");
    var back = _getElement("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

/*=============================================================================
おもて面の表示処理

概要
    おもて面を表示する
引数
    event : イベント情報
戻値
    なし
=============================================================================*/
function showFront(event)
{
    //検索条件を保存
    _commitPrefs();
    
    //おもて面の表示
    var front = _getElement("front");
    var back = _getElement("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}


/*=============================================================================
ジャンル選択時処理

概要
    うら面でジャンルを選択した際に呼び出す関数。<SELECT>タグのonchangeに設定すること。
引数
    なし
戻値
    なし
=============================================================================*/
function _onGenreChange(aIndex)
{
    var ele = _getElement("genreSelect" + aIndex);
    _writeSrchTypeList(aIndex, ele.value);
}


/*=============================================================================
保存済みの主データクリア処理

概要
    プリファレンスに保存した主データをクリアする。
引数
    なし
戻値
    なし
=============================================================================*/
function _clearDataArray()
{
    _clearStrArrayOfPref(__const_key_data);
}


/*=============================================================================
主データの保存処理

概要
    プリファレンスに主データを保存する。
引数
    aDataArray    : 主データを格納した文字列配列
戻値
    なし
=============================================================================*/
function _saveDataArray(aDataArray)
{
    _saveStrArrayToPref(aDataArray, __const_key_data);
}


/*=============================================================================
主データの読み出し処理

概要
    プリファレンスに保存されている主データを読み出して返す。
引数
    なし
戻値
    主データが格納された文字列配列
=============================================================================*/
function _loadDataArray()
{
    return _loadStrArrayFromPref(__const_key_data);
}


/*=============================================================================
ジャンルリストのHTMLへの書き出し

概要
    ジャンル選択用タグをHTMLへ書き出す。
引数
    aIndex    : 条件インデックス
戻値
    なし
=============================================================================*/
function _writeGenreList(aIndex)
{
var i;
    
    var labels = __const_genre_label_array;
    var str = "<select type='select' id='genreSelect" + aIndex + "' onchange='_onGenreChange(" + aIndex + ");'>";
    for ( i = 0 ; i < labels.length ; i++ )
    {
        str += "<option value=" + i + ">" + labels[i] + "</option>";
    }
    str += "</select>";
    var ele = _getElement("genreSelector" + aIndex);
    _removeAllChildren(ele);
    ele.innerHTML = str;
}


/*=============================================================================
検索タイプリストのHTMLへの書き出し

概要
    検索タイプ選択用タグをHTMLへ書き出す。
引数
    aIndex    : 条件インデックス
    aGenre    : ジャンル値
戻値
    なし
=============================================================================*/
function _writeSrchTypeList(aIndex, aGenre)
{
var i;

    var gnr = ( null == aGenre ) ? __const_default_genre : aGenre;
    var list = __const_srchpram_srchtype_array[gnr];
    var labels = __const_srchtype_label_array[gnr];
    var str = "<select type='select' id='srchtypeSelect" + aIndex + "'>";
    for ( i = 0 ; i < list.length ; i++ )
    {
        str += "<option value=" + i + ">" + labels[i] + "</option>";
    }
    str += "</select>";
    var ele = _getElement("srchtypeSelector" + aIndex);
    _removeAllChildren(ele);
    ele.innerHTML = str;
}


/*=============================================================================
直下要素の全削除

概要
    要素直下（その子孫を含む）を削除する
引数
    aNode    : 親要素
戻値
    なし
=============================================================================*/
function _removeAllChildren(aNode)
{
    for ( var i = aNode.childNodes.length-1 ; i >= 0 ; i-- )
    {
        var cn = aNode.childNodes[i];
        if ( cn.onclick != null ) {
            cn.onclick = null;
        }
        else if ( cn.onchange != null ) {
            cn.onchange = null;
        }
        _removeAllChildren(cn);
        aNode.removeChild(cn);
        delete cn;
    }
}

/*=============================================================================
Amazonページ表示処理

概要
    ブラウザでAmazonページを表示する。
引数
    aAsin    : 表示品のASINコード。
戻値
    なし
=============================================================================*/
function _openAmazonPage(aAsin)
{
    widget.openURL(__const_open_amazon_page_url_prefix + aAsin + __const_open_amazon_page_url_suffix);
}


/*=============================================================================
ステータスの表示処理

概要
    おもて面にステータスメッセージを表示
引数
    aMessage    : 表示するメッセージ文字列 (nullで変更せず)
    aLed        : LED表示 (null=変更せず, 0=OFF, 1=ON, 2=WARN, 3=ERR)
戻値
    なし
=============================================================================*/
function _displayStatus(aMessage, aLed)
{
    var stsarea = _getElement("statusArea");
    if ( aMessage != null ) {
        _removeAllChildren(stsarea);
        stsarea.innerHTML = aMessage;
    }
    if ( aLed != null ) {
        stsarea.className = __const_message_color_array[aLed];
    }
}


/*=============================================================================
最終更新日の表示

概要
    おもて面のステータスメッセージ欄に最終更新日を表示
引数
    aLed        : LED表示 (null=変更せず, 0=OFF, 1=ON, 2=WARN, 3=ERR)
戻値
    なし
=============================================================================*/
function _displayLastUpdatedAt(aLed)
{
    _displayStatus(__const_msg_last_updated_at + _date2Text(__dataLastUpdateDate), aLed);
}


/*=============================================================================
主データ表示処理

概要
    <TR>タグで書かれた表示用データをおもて面に表示する。（<TABLE>タグを構成する）
引数
    aTableRowHtmlDataArray    : 表示用<TR>ソース文字列配列
戻値
    なし
=============================================================================*/
function _displayData(aTableRowHtmlDataArray)
{
var i;

    //テーブルを作成
    var str = "<table class='dataList'>";
    if ( aTableRowHtmlDataArray )
    {
        for ( i = 0 ; i < aTableRowHtmlDataArray.length ; i++ )
        {
            if ( __const_list_row_count == i ) break;    //上限行数に達したら終了
            str += aTableRowHtmlDataArray[i];
        }
    }
    str += "</table>";

    //テーブルを表示
    var dataarea = _getElement("dataArea");
    _removeAllChildren(dataarea);
    dataarea.innerHTML = str;
}


/*=============================================================================
検索キーワードクリア処理

概要
    プリファレンスに保存した検索条件をクリアする。
引数
    なし
戻値
    なし
=============================================================================*/
function _clearConditionArrays()
{
    _setLocalPkey(null, __const_lpkey_captionValue);
    _setLocalPkey(null, __const_lpkey_limitValue);
    _setLocalPkey(null, __const_lpkey_colorValue);
    _clearStrArrayOfPref(__const_key_sc_genre);
    _clearStrArrayOfPref(__const_key_sc_srchtype);
    _clearStrArrayOfPref(__const_key_sc_keyword);
    _clearStrArrayOfPref(__const_key_sc_exclude);
}


/*=============================================================================
検索キーワード保存処理

概要
    プリファレンスに検索条件を保存する。
引数
    aKeywordArray    ：検索キーワード文字列配列
戻値
    なし
=============================================================================*/
function _saveConditionArrays(aCaption, aLimit, aColor, aGenreArray, aSrchTypeArray, aKeywordArray, aExcludeArray)
{
    _setLocalPkey(aCaption, __const_lpkey_captionValue);
    _setLocalPkey(aLimit, __const_lpkey_limitValue);
    _setLocalPkey(aColor, __const_lpkey_colorValue);
    _saveStrArrayToPref(aGenreArray, __const_key_sc_genre);
    _saveStrArrayToPref(aSrchTypeArray, __const_key_sc_srchtype);
    _saveStrArrayToPref(aKeywordArray, __const_key_sc_keyword);
    _saveStrArrayToPref(aExcludeArray, __const_key_sc_exclude);
}


/*=============================================================================
検索キーワード読み出し処理

概要
    プリファレンスから検索条件を読み出す。
引数
    なし
戻値
    検索キーワード文字列配列
=============================================================================*/
function _loadConditionOfCaption()
{
    return _getLocalPkey(__const_lpkey_captionValue);
}
function _loadConditionOfLimit()
{
    return _getLocalPkey(__const_lpkey_limitValue);
}
function _loadConditionOfColor()
{
    return _getLocalPkey(__const_lpkey_colorValue);
}
function _loadConditionOfGenreArray()
{
    var a = _loadStrArrayFromPref(__const_key_sc_genre);
    if ( null == a ) a = new Array();
    while ( a.length < __const_condition_count ) a[a.length] = __const_default_genre;
    return a;
}
function _loadConditionOfSrchTypeArray()
{
    var a = _loadStrArrayFromPref(__const_key_sc_srchtype);
    if ( null == a ) a = new Array();
    while ( a.length < __const_condition_count ) a[a.length] = __const_default_srchtype;
    return a;
}
function _loadConditionOfKeywordArray()
{
    var a = _loadStrArrayFromPref(__const_key_sc_keyword);
    if ( null == a ) a = new Array();
    while ( a.length < __const_condition_count ) a[a.length] = null;
    return a;
}
function _loadConditionOfExcludeArray()
{
    var a = _loadStrArrayFromPref(__const_key_sc_exclude);
    if ( null == a ) a = new Array();
    while ( a.length < __const_condition_count ) a[a.length] = null;
    return a;
}


/*=============================================================================
WebサービスからXMLデータを取り出す処理

概要
    Webサービスにアクセスして、指定の検索条件の検索結果を得る。
引数
    aSrchType    : 検索キーワードの適用方法値（本ファイル冒頭で__const定義された値による）
    aKeyword    : 検索キーワード
    aGenreValue    : ジャンル値（本ファイル冒頭で__const定義された値による）
    aPageNumner : 取得ページ番号（１）
    aSuccessHandler : 成功時処理用関数
    aErrorHandler       : 失敗時処理用関数
    aErrorHandler       : 受信中オフライン時処理用関数
戻値
    XMLデータオブジェクト
=============================================================================*/
function _fetchXmlDataByKeyword(aSrchType, aKeyword, aGenreValue, aPageNumber, aSuccessHandler, aErrorHandler, aOfflineHandler)
{
    //ハンドラを保存
    __requestSuccessHandler = aSuccessHandler;
    __requestErrorHandler = aErrorHandler;
    __requestOfflineHandler = aOfflineHandler;

    //検索キーワードの適用方法を取得
    var srchtype = __const_srchpram_srchtype_array[aGenreValue][aSrchType];

    //REST問い合わせ用URLを作成（プラグイン利用）
    var params_str =
        "ItemPage" + __const_sprt + aPageNumber + __const_sprt +
        "Operation" + __const_sprt + "ItemSearch" + __const_sprt +
        "AssociateTag" + __const_sprt + __const_aws_ascid + __const_sprt +
        "ResponseGroup" + __const_sprt + "Request,Medium,Offers" + __const_sprt +
        srchtype + __const_sprt + aKeyword  + __const_sprt +
        "SearchIndex" + __const_sprt + __const_srchparam_srchidx_array[aGenreValue] + __const_sprt +
        "Sort" + __const_sprt + __const_srchparam_sort_array[aGenreValue];
    var query_url = NewReleasePlugIn.searchAmazon(params_str, __const_sprt);

    //Webサービスに情報を問い合わせる。
    //※リクエストオブジェクトは安全のため、毎回生成
    __request = _createXMLHttpRequest();
    __request.onreadystatechange = _handleHttpEvent;
     __request.open("GET", query_url, true);
    __request.send(null);
}

/*=============================================================================
非同期通信の状態ハンドラ

概要
    XMLHTTPRequest.onreadystatechange にセットされ、通信状態の変化時に呼び出される。
=============================================================================*/
function _handleHttpEvent()
{
    if (4 == __request.readyState )
    {
        //成功時処理を呼び出し
        if ( 200 == __request.status )
        {
            __requestSuccessHandler(__request.responseXML);
            __request = null;
        }
        //失敗時処理を呼び出し
        else if ( 0 == __request.status ) {
            __requestOfflineHandler();
            __request = null;
            __dataLastUpdateDate.setTime(0);
        }
        //失敗時処理を呼び出し
        else {
            __requestErrorHandler();
            __request = null;
            __dataLastUpdateDate.setTime(0);
        }
    }
}


/*=============================================================================
XMLデータからエラー情報を取得

概要
    XMLデータ中の、"<Errors>"情報を得る。
引数
    aXmlData    : エラー情報を取り出すXMLデータオブジェクト
戻値
    エラー情報配列
=============================================================================*/
function _getErrorFromXmlData(aXmlData)
{
var i, r;
var error_infos = new Array();

    var errors = aXmlData.getElementsByTagName('Errors');
    
    r = 0;
    for ( i = 0 ; i < errors.length ; i++ ) {
        var err_code = errors(0).childNodes(0).childNodes(0).childNodes(0).nodeValue;
        var err_msg = errors(0).childNodes(0).childNodes(1).childNodes(0).nodeValue;
        if ( err_code != __const_amazon_error_NoExactMatches )
        {
            error_infos[r * 2] = "<tr><td>*" + err_code;
            error_infos[r * 2 + 1] = "<br>____" + err_msg + "</td></tr>";
            //alert(err_code + ":" + err_msg); //***
            r += 1;
        }
    }
        
    return error_infos;    
}


/*=============================================================================
XMLデータからページ数を取得

概要
    XMLデータ中の、"TotalPages"情報を得る。
引数
    aXmlData    : ページ数を取り出すXMLデータオブジェクト
戻値
    ページ数
=============================================================================*/
function _getPageCountFromXmlData(aXmlData)
{
    var total_pages = aXmlData.getElementsByTagName('TotalPages');
    if ( total_pages.length > 0 )
    {
        return parseInt(total_pages[0].childNodes.item(0).nodeValue);
    }
    else return 0;    
}


/*=============================================================================
XMLデータを<TR>タグコードに変換する処理

概要
    指定したXMLデータの品目を、<TABLE>タグで使用可能な<TR>タグ群に変換する。
引数
    aXmlData            : XMLデータ
    aGenreValue            : ジャンル値（本ファイル冒頭で__const定義された値による）
    aXday                : 未来日付制限日数
    aExcludeWords    : 不要キーワード群（スペースまたはカンマ区切り）
戻値
    <TR>～</TR>を１行に収めた文字列配列
=============================================================================*/
function _makeTableRowHTMLArrayFromXmlData(aXmlData, aGenreValue, aXdays, aExcludeWords)
{
var i, idx;

    //リミット日付を作成    
    var date_limit_str = _getXdaysAfterStr(aXdays);

    //情報をHTMLテーブルの作成ループ
    var html_array = new Array();
    var items = aXmlData.getElementsByTagName('Item');
    var count = items.length;
    idx = 0;
    for (i = 0 ; i < count ; i++)
    {
        //属性ノードを取得
        var atb = items[i].getElementsByTagName('ItemAttributes');
        //データ日付ノードを取得
        //_displayNodeTree(atb[0]);
        var pubdateElement = atb[0].getElementsByTagName(__const_srchresult_pubdate_array[aGenreValue]);
        //日付データを作成
        var pubdate, check_pubdate;
        if ( pubdateElement.length )
        {
            pubdate = pubdateElement[0].childNodes.item(0).nodeValue;
            check_pubdate = pubdate;
            if ( pubdate.length == 4 )
            {
                pubdate += "-**-**";
                check_pubdate += "-01-01";
            }
            else if ( pubdate.length == 7 )
            {
                pubdate += "-**";
                check_pubdate += "-01";
            }
        }
        else {
            var salesrankElement = items[i].getElementsByTagName('SalesRank');
            if ( salesrankElement.length )
            {
                pubdate = __const_salesrank_display_prefix + (__const_salesrank_display_blank + salesrankElement.item(0).firstChild.nodeValue).slice((__const_salesrank_display_prefix.length + __const_salesrank_display_blank.length) * -1);
                check_pubdate = null;            }
            else {
                pubdate = "****-**-**";
                check_pubdate += "0001-01-01";
            }
        }
        //未来日付制限に引っかかったら、本データは処理しない
        if ( pubdateElement.length && date_limit_str < check_pubdate ) continue;
        
        //見出し取得
        var title = atb[0].getElementsByTagName('Title')[0].childNodes.item(0).nodeValue;
        //対象外文字列が含まれていたら、本データは処理しない
        if ( _checkIncludeWords(title, aExcludeWords) ) continue;
        
        var asin = items[i].getElementsByTagName('ASIN')[0].childNodes.item(0).nodeValue;
        var url = items[i].getElementsByTagName('DetailPageURL')[0].childNodes.item(0).nodeValue;
        str = __const_tableRowHtmlPrefix + pubdate + "</td>";
        str += "<td class='dataListTitleCell' nowrap=false>";

        str += "<span onclick='_openAmazonPage(\"" + asin + "\");'>";
        str += title;
        str += "</span>";

        str += "</td></tr>";
        html_array[idx++] = str;
    }

    return html_array;
}
    
    
/*=============================================================================
リスト更新処理

概要
    プリファレンスに保存された検索条件を使用して、表示情報を最新版に更新する。
    _refreshList() の呼び出しで開始する。
引数
    aWaitMills    : 起動タイマ値（ミリ秒。この指定時間経過後に実際に更新を開始する）
戻値
    なし
=============================================================================*/
//処理用変数
var __v_refreshList_lastCallInterval = null; //呼び出し待ち時間
var __v_refreshList_processing = false;        //処理中フラグ
var __v_refreshList_interval_timer = null;  //実行待ちタイマー
var __v_refreshList_start_timer = null;        //処理待ちタイマー
var __v_refreshList_limit;                  //日付範囲（リミット日数値)
var __v_refreshList_genreArray;                //ジャンル群保持用
var __v_refreshList_srchtypeArray;            //検索タイプ群保持用
var __v_refreshList_keywordArray;            //キーワード群保持用
var __v_refreshList_excludeArray;        //不要キーワード群保持用
var __v_refreshList_tableRowHTMLArray;        //結果保持用
var __v_refreshList_nowGenre;                //現在処理中キーワード
var __v_refreshList_nowSrchType;            //現在処理中キーワード
var __v_refreshList_nowKeyword;                //現在処理中キーワード
var __v_refreshList_nowExclude;            //現在処理中キーワード
var __v_refreshList_nowKeywordPageCount;    //現在処理中キーワードでの結果ページ数
var __v_refreshList_nowKeywordPageNumber;    //現在処理中キーワードでの処理中ページ番号
var __v_refreshList_nowKeywordRowArray;        //現在処理中キーワードでの結果保持用
var __v_refreshList_receiveErrorArray;        //受信したエラー情報
//処理開始関数
function _refreshList()
{
    //処理中であれば処理しない
    if ( __v_refreshList_processing ) {
        return;
    }

    //オフラインであれば処理しない
    if ( false == __onlined ) {
        //次回のチェック処理をタイマでセット
        _sub_refreshList_setNextCheck();
        return;
    }

    //処理待ちならばタイマを停止する
    if ( null != __v_refreshList_start_timer ) {
        clearTimeout(__v_refreshList_start_timer);
        __v_refreshList_start_timer = null;
    }
    
    //最終更新日時からの経過をチェック
    var nowD = new Date();
    if ( ( __dataLastUpdateDate.getTime() + __const_refresh_interval ) <= nowD.getTime() ) {
        //最終更新日時を記録する
        __dataLastUpdateDate = nowD;
        //呼び出し間隔タイマーをリセット
        __v_refreshList_interval_timer = null;
        //開始処理を起動
        __v_refreshList_start_timer = setTimeout("_sub_refreshList_start();", 1);
    }
    else {
        //メッセージ欄を最終更新日表示にセット
        _displayLastUpdatedAt(0);
        //次回のチェック処理をタイマでセット
        _sub_refreshList_setNextCheck();
    }
    
}
//処理開始関数 次回起動処理
function _sub_refreshList_setNextCheck(aInterval)
{
    var itv = ( aInterval == null ) ? __const_check_interval : aInterval;
    if ( __v_refreshList_interval_timer != null )
    {
        clearTimeout(__v_refreshList_interval_timer);
    }
    __v_refreshList_interval_timer = setTimeout("_refreshList();", itv);
}
//開始処理
function _sub_refreshList_start()
{
    //処理中フラグをONにする
    __v_refreshList_processing = true;
    //タイマをクリアする
    __v_refreshList_start_timer = null;

    //更新中表示を行う
    _displayStatus(__const_msg_updating, 1);
    
    //現在のデータをクリアする
    _clearDataArray();

    //検索条件を取り出す
    __v_refreshList_limit = _loadConditionOfLimit();
    __v_refreshList_genreArray = _loadConditionOfGenreArray();
    __v_refreshList_srchtypeArray = _loadConditionOfSrchTypeArray();
    __v_refreshList_keywordArray = _loadConditionOfKeywordArray();
    __v_refreshList_excludeArray = _loadConditionOfExcludeArray();
    //検索キーワードが未登録ならば、後処理に移行
    if ( null == __v_refreshList_keywordArray || 0 == __v_refreshList_keywordArray.length )
    {
        _displayStatus(" ", 0);
        __successed = null;
        _sub_refreshList_term();
    }
    //検索キーワードが取り出せたら、主処理に移行
    else {
        __v_refreshList_tableRowHTMLArray = new Array();
        setTimeout("_sub_refreshList_proc_start();", __const_wait_interval);    //表示更新を行わせるため、タイマを使用。
    }
}
//主処理スタート
function _sub_refreshList_proc_start()
{
    //キーワードがなければ主処理エンドへ遷移
    if ( 0 == __v_refreshList_keywordArray.length )
    {
        setTimeout("_sub_refreshList_proc_end();", 1);
    }
    //キーワードがあれば・・・
    else {
        //条件を１つ取り出し
        __v_refreshList_nowGenre = __v_refreshList_genreArray.shift();
        __v_refreshList_nowSrchType = __v_refreshList_srchtypeArray.shift();
        __v_refreshList_nowKeyword = __v_refreshList_keywordArray.shift();
        __v_refreshList_nowExclude = __v_refreshList_excludeArray.shift();
        //キーワードがnullなら本関数を再実行
        if ( null == __v_refreshList_nowKeyword || "" == __v_refreshList_nowKeyword )
        {
            setTimeout("_sub_refreshList_proc_start();", 1);
        }
        //キーワードがあれば主処理開始
        else {
            //現在処理中キーワードの処理中ページ番号を１にセット
            __v_refreshList_nowKeywordPageNumber = 1;
            //現在処理中キーワードでの結果保持配列を初期化
            __v_refreshList_nowKeywordRowArray = new Array();
            //起動
            setTimeout("_sub_refreshList_proc_main();", 1);
        }
    }
}
//主処理メイン
function _sub_refreshList_proc_main()
{
    //XMLデータを取得
    _fetchXmlDataByKeyword(__v_refreshList_nowSrchType, __v_refreshList_nowKeyword, __v_refreshList_nowGenre, __v_refreshList_nowKeywordPageNumber, _sub_refreshList_proc_success, _sub_refreshList_proc_http_error, _sub_refreshList_proc_offline);
}
//データ受信成功時処理
function _sub_refreshList_proc_success(aXmlData)
{
//_displayNodeTree(aXmlData.childNodes[0].childNodes[1]); //***

    //エラーチェック
    __v_refreshList_receiveErrorArray = _getErrorFromXmlData(aXmlData);
    if ( __v_refreshList_receiveErrorArray.length > 0 )
    {
        setTimeout("_sub_refreshList_proc_receive_error();", 1);
        return;
    }

    //ページ件数を取得
    __v_refreshList_nowKeywordPageCount = _getPageCountFromXmlData(aXmlData);
    //データ表示用のHTML行データを作成（未来日付制限、不要キーワード排除付き）
    var table_row_array = _makeTableRowHTMLArrayFromXmlData(aXmlData, __v_refreshList_nowGenre, __v_refreshList_limit, __v_refreshList_nowExclude);
    //現在キーワード結果を結合
    __v_refreshList_nowKeywordRowArray = __v_refreshList_nowKeywordRowArray.concat(table_row_array);
    
    //処理ページ番号を更新
    __v_refreshList_nowKeywordPageNumber++;
    
    //全ページを処理したか、10ページを超えたか（2011-10-26新仕様）、現在キーワードの結果が表示件数に達していたら、
    //検索結果保存配列にマージして次のキーワード処理へ
    if ( __v_refreshList_nowKeywordPageNumber > __v_refreshList_nowKeywordPageCount ||
        __v_refreshList_nowKeywordPageNumber > 10 ||
        __const_list_row_count <= __v_refreshList_nowKeywordRowArray.length )
    {
        //表示件数で足切り
        __v_refreshList_nowKeywordRowArray.length = __const_list_row_count;
        //検索結果保存配列にマージ
        __v_refreshList_tableRowHTMLArray = _margeDistinctArray(__v_refreshList_tableRowHTMLArray, __v_refreshList_nowKeywordRowArray);
        //Amazon仕様の待ち時間後に主処理スタートを開始
        setTimeout("_sub_refreshList_proc_start();", __const_amazon_interval);
    }
    //達していなければ、次ページを処理
    else {
        //Amazon仕様の待ち時間後にに主処理メインを開始
        setTimeout("_sub_refreshList_proc_main();", __const_amazon_interval);
    }
}
//主処理エンド
function _sub_refreshList_proc_end()
{
var i;
    //データが存在したら整形処理
    if ( __v_refreshList_tableRowHTMLArray.length > 0 )
    {
        //データを日付の降順でソート（先頭に日付データがあるため、データをソートして順序を反転させればOK）
        __v_refreshList_tableRowHTMLArray = __v_refreshList_tableRowHTMLArray.sort();
        __v_refreshList_tableRowHTMLArray = __v_refreshList_tableRowHTMLArray.reverse();
        //件数で足切り
        if ( __v_refreshList_tableRowHTMLArray.length > __const_list_row_count )
        {
            __v_refreshList_tableRowHTMLArray.length = __const_list_row_count;
        }
        //SalesRankを含むとき（1行目の1桁目が"R"）は、SalesRank部分のみ再度昇順にする。
        if ( __v_refreshList_tableRowHTMLArray[0][__const_tableRowHtmlPrefix.length] == __const_salesrank_display_prefix )
        {
            var last = 0;
            var wa = new Array();
            for ( i = 0 ; i < __v_refreshList_tableRowHTMLArray.length ; i++ )
            {
                if ( __v_refreshList_tableRowHTMLArray[i][__const_tableRowHtmlPrefix.length] == __const_salesrank_display_prefix )
                {
                    wa[i] = __v_refreshList_tableRowHTMLArray[i];
                    last = i;
                }
                else break;
            }
            wa.reverse();
            for ( i = 0 ; i <= last ; i++ )
            {
                __v_refreshList_tableRowHTMLArray[i] = wa[i];
            }
        }
    }
    //保存
    _saveDataArray(__v_refreshList_tableRowHTMLArray);
    //最終更新日を表示
    _displayLastUpdatedAt(0);
    //後処理へ遷移
    __successed = true;
    setTimeout("_sub_refreshList_term();", 1);
}
//後処理
function _sub_refreshList_term()
{
    //データを表示
    _displayData(__v_refreshList_tableRowHTMLArray);

    //更新処理を終了状態に移行
    __v_refreshList_processing = false;

    //チェック処理をセット
    _sub_refreshList_setNextCheck();
}
//HTTPエラー時処理
function _sub_refreshList_proc_http_error()
{
    _displayStatus( "Error:" + __request.status + ". Will retry within " + (__const_check_interval / 1000) + " seconds.", 3 );
    __v_refreshList_tableRowHTMLArray = null;
    __successed = false;

    //更新処理を終了状態に移行
    __v_refreshList_processing = false;

    //チェック処理をセット
    _sub_refreshList_setNextCheck();
}
//オフライン時処理
function _sub_refreshList_proc_offline()
{
    _displayStatus( __const_msg_comm_retrying, 2 );
    __v_refreshList_tableRowHTMLArray = null;
    __successed = false;

    //更新処理を終了状態に移行
    __v_refreshList_processing = false;

    //チェック処理を10秒後にセット
    _sub_refreshList_setNextCheck(__const_check_interval_at_offline);
}
//エラー受信時処理
function _sub_refreshList_proc_receive_error()
{
    _displayData( __v_refreshList_receiveErrorArray );
    _displayStatus( "ERROR", 3 );
    __v_refreshList_tableRowHTMLArray = null;
    __successed = false;

    //更新処理を終了状態に移行
    __v_refreshList_processing = false;

    //チェック処理を10秒後にセット
    _sub_refreshList_setNextCheck();
}


/*=============================================================================
リスト更新中止処理

概要
    タイマ付きで起動されたリスト更新処理が、まだ実際の起動前であれば中止します。
引数
    なし
戻値
    停止成功（あらかじめ停止していた場合も含む）ならば true、失敗ならば false
=============================================================================*/
function _abortRefresh()
{
    //処理中であれば失敗を返す
    if ( __v_refreshList_processing ) return false;
    
    //処理待ちならばタイマを停止する
    if ( null != __v_refreshList_start_timer )
    {
        clearTimeout(__v_refreshList_start_timer);
        __v_refreshList_start_timer = null;
    }
    if ( null != __v_refreshList_interval_timer )
    {
        clearTimeout(__v_refreshList_interval_timer);
        __v_refreshList_start_timer = null;
    }
    //更新処理は停止状態となったので、成功を返す
    return true;
}



/*=============================================================================
うら面（設定画面）表示処理

概要
    うら面の設定画面を表示します。
引数
    なし
戻値
    なし
=============================================================================*/
function _showPrefs()
{
    //情報更新処理を停止（失敗したら終了）
    if ( ! _abortRefresh() ) return;

    //ひっくり返し処理（裏面へ）
    var front = _getElement("front");
    var back = _getElement("back");
    widget.prepareForTransition("ToBack");
    front.style.display="none";
    back.style.display="block";
    setTimeout ('widget.performTransition();', 1);

    //iボタンの背景画像を消去
    _getElement('fliprollie').style.display = 'none';
}


/*=============================================================================
うら面（設定画面）決定処理

概要
    うら面の設定画面で指定された各種情報をプリファレンスに保存し、更新処理を実行して
    おもて面を表示します。
引数
    なし
戻値
    なし
=============================================================================*/
function _commitPrefs()
{
var i;

    //情報の保存
    //※保存する値を準備
    var caption = _getElement("captionEdit").value;
    var limit = _getElement("limitSelect").value;
    var color = _getElement("colorSelect").value;
    var genre_array = new Array();
    var srchtype_array = new Array();
    var srchtype_array = new Array();
    var keyword_array = new Array();
    var exclude_array = new Array();
    var idx = 0;
    for ( i = 0 ; i < __const_condition_count ; i++ )
    {
        genre_array[idx] = _getElement("genreSelect"+i).value;
        srchtype_array[idx] = _getElement("srchtypeSelect"+i).value;
        keyword_array[idx] = _getElement("keywordEdit"+i).value;
        exclude_array[idx] = _getElement("excludeEdit"+i).value;
        idx++;
    }
    //※整形
    if ( null == caption ) caption = "";
    if ( null == limit ) limit = __const_default_limit;
    if ( null == color ) color = __const_default_color;
    //※保存
    _saveConditionArrays(caption, limit, color, genre_array, srchtype_array, keyword_array, exclude_array);
    
    //表面の項目表示を更新
    _getElement("captionArea").innerText = caption;

    //情報更新処理を実行
    __dataLastUpdateDate.setTime(0);
    _refreshList();

    //ひっくり返し処理（表面へ）
    var front = _getElement("front");
    var frontImage = _getElement("frontImage");
    var go_amazon = _getElement("goAmazon");
    var back = _getElement("back");
    back.style.display = "none";
    frontImage.className = color;
    front.style.display = "block";
    go_amazon.className = color;
    widget.prepareForTransition("ToFront");
    setTimeout ('widget.performTransition();', 1);
}

/*=============================================================================
Amazonサイトの呼び出し

概要
    Amazonサイトをブラウザで表示する。
引数
    なし
戻値
    なし
=============================================================================*/
function showAmazonSite()
{
    widget.openURL(__const_show_amazon_site_url);
}


/*=============================================================================
配布サイトの呼び出し

概要
    配布サイトをブラウザで表示する。
引数
    なし
戻値
    なし
=============================================================================*/
function showMySite()
{
    widget.openURL(__const_show_my_site_url);
}


/*=*=*=========================================================================
以下、Widgetのユーザーインターフェース処理関数群
※サンプルコードをそのまま使用。
=============================================================================*/


// PREFERENCE BUTTON ANIMATION (- the pref flipper fade in/out)

var flipShown = false;        // a flag used to signify if the flipper is currently shown or not.


// A structure that holds information that is needed for the animation to run.
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};


// mousemove() is the event handle assigned to the onmousemove property on the front div of the widget. 
// It is triggered whenever a mouse is moved within the bounds of your widget.  It prepares the
// preference flipper fade and then calls animate() to performs the animation.

function mousemove (event)
{
    if (!flipShown)            // if the preferences flipper is not already showing...
    {
        if (animation.timer != null)            // reset the animation timer value, in case a value was left behind
        {
            clearInterval (animation.timer);
            animation.timer  = null;
        }
        
        var starttime = (new Date).getTime() - 13;         // set it back one frame
        
        animation.duration = 500;                                                // animation time, in ms
        animation.starttime = starttime;                                        // specify the start time
        animation.firstElement = _getElement ('flip');        // specify the element to fade
        animation.timer = setInterval ("animate();", 13);                        // set the animation function
        animation.from = animation.now;                                            // beginning opacity (not ness. 0)
        animation.to = 1.0;                                                        // final opacity
        animate();                                                                // begin animation
        flipShown = true;                                                        // mark the flipper as animated
    }
}

// mouseexit() is the opposite of mousemove() in that it preps the preferences flipper
// to disappear.  It adds the appropriate values to the animation data structure and sets the animation in motion.

function mouseexit (event)
{
    if (flipShown)
    {
        // fade in the flip widget
        if (animation.timer != null)
        {
            clearInterval (animation.timer);
            animation.timer  = null;
        }
        
        var starttime = (new Date).getTime() - 13;
        
        animation.duration = 500;
        animation.starttime = starttime;
        animation.firstElement = _getElement ('flip');
        animation.timer = setInterval ("animate();", 13);
        animation.from = animation.now;
        animation.to = 0.0;
        animate();
        flipShown = false;
    }
}


// animate() performs the fade animation for the preferences flipper. It uses the opacity CSS property to simulate a fade.

function animate()
{
    var T;
    var ease;
    var time = (new Date).getTime();
        
    
    T = limit_3(time-animation.starttime, 0, animation.duration);
    
    if (T >= animation.duration)
    {
        clearInterval (animation.timer);
        animation.timer = null;
        animation.now = animation.to;
    }
    else
    {
        ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
        animation.now = computeNextFloat (animation.from, animation.to, ease);
    }
    
    animation.firstElement.style.opacity = animation.now;
}


// these functions are utilities used by animate()

function limit_3 (a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
    return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events

function enterflip(event)
{
    _getElement('fliprollie').style.display = 'block';
}

function exitflip(event)
{
    _getElement('fliprollie').style.display = 'none';
}



//==== Startup =====

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}
